import React, { forwardRef, useEffect, useImperativeHandle, useRef, useState } from 'react';
import { ValidationEventType } from '../models/validation/ValidationEventType';
import { ValidationOverride } from '../models/validation/ValidationOverride';
import { ValidationResult } from '../models/validation/ValidationResult';
import { formValidator } from './helpers/FormValidator';
import { formbuilder } from '../utils/FormBuilderInitializer';
import { validationUtil } from '../utils/common/ValidationUtil';
import { schemaFromConfig } from '../utils/schema/schemaFromConfig';
import { useIsMounted } from '../hooks/useIsMounted';
const EmptyGroupContainer = props => {
    return React.createElement("div", { key: `emptygroup-${props.itemId}` }, props.children);
};
export const Form = forwardRef((props, ref) => {
    // verify that the formbuilder is set up correctly
    formbuilder.verify();
    const isMounted = useIsMounted();
    const [item, setItem] = useState(props.item);
    const [schema, setSchema] = useState(props.dynamicSchema != null ? undefined : props.schema);
    const dynamicSchema = useRef();
    const [groupings, setGroupings] = useState();
    const [groupsHandled, setGroupsHandled] = useState(false);
    const [validationResults, setValidationResults] = useState(props.validationResult ?? {});
    const validateItem = async (formItem, schema, on = ValidationEventType.Manual, prop) => {
        let selectedSchema = dynamicSchema.current ?? schema;
        switch (props.validationOverride) {
            case undefined:
            case ValidationOverride.None:
                let results = await formValidator.validate(formItem, prop, selectedSchema, on, validationResults, props.validationResultPrefix);
                setValidationResults({ ...results });
                return validationUtil.validated(results) ? ValidationResult.Success : ValidationResult.Failed;
            case ValidationOverride.Ignore:
                setValidationResults({});
                return ValidationResult.Success;
            case ValidationOverride.Continue:
                let results2 = await formValidator.validate(formItem, prop, selectedSchema, on, validationResults, props.validationResultPrefix);
                setValidationResults({ ...results2 });
                return validationUtil.validated(results2) ? ValidationResult.Success : ValidationResult.FailedDontBlock;
            default:
                return ValidationResult.Success;
        }
    };
    useImperativeHandle(ref, () => ({
        getItem: () => item,
        validateItem: (type) => validateItem(item, props.schema, type, undefined)
    }), [item, props.schema]);
    const getDynamicSchemaKey = (it) => {
        let dynamicKey = props.dynamicSchema?.dynamicKey;
        if (dynamicKey == null)
            return undefined;
        return dynamicKey(it ?? item);
    };
    const loadSchema = async (it, force) => {
        if (schema !== undefined && props.dynamicSchema?.dynamicKey == null && !force)
            return;
        // let schemaMerged = await fetchSchema<T>(props.dynamicSchema?.customFormType ?? props.itemType, props.formItemConfigOverrides, props.propertyOverrides, getDynamicSchemaKey(it));
        let schemaMerged = await schemaFromConfig(props.dynamicSchema?.schemaConfig ?? props.schemaConfig, props.formItemConfigOverrides, props.propertyOverrides, getDynamicSchemaKey(it));
        if (props.dynamicSchema?.schemaConfig != null) {
            dynamicSchema.current = schemaMerged;
        }
        if (!isMounted())
            return;
        setSchema(schemaMerged);
    };
    useEffect(() => {
        loadSchema();
    }, [props.schemaConfig]);
    useEffect(() => {
        // if (props.propertyOverrides == null) return;
        loadSchema(undefined, true);
    }, [props.propertyOverrides]);
    const GroupContainer = props.groupContainer ?? EmptyGroupContainer;
    useEffect(() => {
        buildGroupings();
    }, [schema, item, validationResults]);
    const buildGroupings = () => {
        if (schema?.options?.groups == null) {
            setGroupsHandled(true);
            return;
        }
        let groups = schema?.options?.groups;
        let groupedProps = {};
        Object.keys(schema.options.properties).forEach(_ => {
            let prop = schema.options.properties[_];
            if (prop.hide?.(item))
                return;
            let groupKey = prop.group ?? "";
            if (groupedProps[groupKey] == null)
                groupedProps[groupKey] = [];
            groupedProps[groupKey].push(_);
        });
        let propertyGroupings = Object.keys(groupedProps).map(_ => {
            let group = Object.keys(groups).length && groups[_] ? groups[_] : { displayName: "Not grouped" }; // TODO: fuck the stupid typescript linter (ewi)
            return {
                groupKey: _,
                displayName: group.displayName,
                groupProps: { ...group.groupProps },
                properties: groupedProps[_],
                validationError: groupValidation(groupedProps[_])
            };
        });
        setGroupings(propertyGroupings);
        setGroupsHandled(true);
    };
    const groupValidation = (properties) => {
        if (validationResults == null)
            return false;
        let error = false;
        properties?.forEach(_ => {
            const result = validationResults[_];
            if (result == null)
                return;
            Object.keys(result).forEach(r => {
                var res = result[r];
                if (res != null && !res.success)
                    error = true;
            });
        });
        return error;
    };
    const onPropertyChange = (property, value) => {
        let clone = { ...item };
        clone[property] = value;
        // setItem(clone);
        // validateItem(clone, schema, ValidationEventType.Change, property);
        var onChanged = schema?.options.properties[property]?.onChanged;
        if (onChanged != null) {
            let result = onChanged(clone);
            if (result?.refreshDynamicSchema) {
                loadSchema(clone);
            }
        }
        setItem(clone);
        validateItem(clone, schema, ValidationEventType.Change, property);
        if (props.onPropertyChange)
            props.onPropertyChange(clone, property, value);
    };
    const Spinner = formbuilder.formItemRender.loadingComponent();
    if (item == null)
        return null;
    if (schema == null)
        return React.createElement(Spinner, { ...props.loadingProps });
    if (!groupsHandled)
        return null;
    const propertyRenderProps = {
        item: item,
        schema: schema.options,
        onChange: onPropertyChange,
        onBlur: (prop) => validateItem(item, schema, ValidationEventType.Blur, prop),
        validationResults: validationResults,
        validationResultPrefix: props.validationResultPrefix,
        propertyOverrides: props.propertyOverrides // this is merged with the schema, and shouldnt be part of the child items (ewi)
    };
    return (React.createElement("div", { className: "formbuilder-formcontainer", key: `${props.keyPrefix}-formbuilder-formcontainer` },
        React.createElement(GroupContainer, { groupings: groupings, key: `${props.keyPrefix}-formbuilder-groupcontainer` }, groupings?.map((_, idx) => {
            let children = formbuilder.formItemRender.properties(propertyRenderProps, _.properties);
            if (children === undefined)
                return null;
            const GroupRender = props.groupRender;
            return GroupRender ? React.createElement(GroupRender, { grouping: _, key: `group-${_.groupKey}-${idx}` }, children) : children;
        })
            ?? formbuilder.formItemRender.properties(propertyRenderProps) // not defining groups here, defaults to all
        )));
});
//# sourceMappingURL=Form.js.map